
; /*********************************************************************
; *
; *             EP7312 specific code
; *
; **********************************************************************
; 
;    When running with Smart Firmware, address 0x0 is usually ROM. 
;    Because we want to replace execption vectors, we use MMU to map 
;    this to RAM. Usually program image starts at 0xc0100000.
;    
; */
        AREA |C$$code|, CODE, READONLY

        CODE32
        EXPORT EP7312_MMU_setup
EP7312_MMU_setup
        MRC     p15, 0, r0, c2, c0          ; r0 = mmu page table base 
        LDR     r1,MMUmask 
        AND     r0,r0,r1
        LDR     r1,MMUentry 
        STR     r1,[r0]
        LDR     r0, =0x00000000
        MCR     p15, 0, r0, c8, c7,0            ; flush the TLB.
        BX     LR
        
MMUentry
        DCD 0x60000c1e
MMUmask
        DCD 0xffffc000
        

; /*********************************************************************
; *
; *             Exception handler table
; *
; **********************************************************************
;    
;    This table is copied to 0x0. This requires RAM at 0x0, which is
;    achieved by calling EP7312_MMU_setup.
;  
; */          
        CODE32
        EXPORT EP7312_Vectors
        IMPORT OS_IRQ_SERVICE
EP7312_Vectors
        LDR           pc,_embResetVec
        LDR           pc,_embUndefVec
        LDR           pc,_embSWIVec
        LDR           pc,_embABPreVec
        LDR           pc,_embABDatVec
        LDR           pc,_embRESRVVec
        LDR           pc,_embIRQVec
        LDR           pc,_embFIQVec
_embResetVec   DCD _embRSThandler
_embUndefVec   DCD _embUNDhandler
_embSWIVec     DCD _embSWIhandler
_embABPreVec   DCD _embABTPhandler
_embABDatVec   DCD _embABTDhandler
_embRESRVVec   DCD _RESRV
_embIRQVec     DCD OS_IRQ_SERVICE
_embFIQVec     DCD _embFIQhandler



; /*********************************************************************
; *
; *             Dummy exception handler
; *
; **********************************************************************
;    
;    All exceptions except IRQ jump here. Feel free to modify this, if
;    you have to service these exceptions. Make sure IRQ handler does
;    execute OS_IRQ_SERVICE in rtosvect.s.
;  
; */        
        CODE32
_embRSThandler
_embUNDhandler
_embSWIhandler
_embABTPhandler
_embABTDhandler
_RESRV
_embFIQhandler
_forever
        B _forever
        END

