
; /********************************************************
; *        SEGGER MICROCONTROLLER SYSTEME GmbH
; *  Solutions for real time microcontroller applications
; *********************************************************
; File        : boot.s
; Purpose     : low level startup code
; --------------END-OF-HEADER----------------------------*/

; ********************************************************
; *        Low level Startup Code for ARM compiler
; ********************************************************
;
; This bases on description given in 
;
;     ARM Software Development Toolkit Version 2.50
;     USER GUIDE
;     Chapter 10
;     Writing Code for ROM
;

; ********************************************************
; *           Define Locations for stacks
; ********************************************************

IRQ_STACK_SIZE  EQU     0x200
SVC_STACK_SIZE  EQU     0x200

; ********************************************************
;             macros for CPU mode switch
; ********************************************************

MODE_BITS       EQU  0x1F            ; Bit mask for mode bits in CPSR
IRQ_MODE        EQU  0x12            ; Interrupt Request mode
SVC_MODE        EQU  0x13            ; Supervisor mode
FLAG_I          EQU  0x80

        CODE32

        MACRO
        mode_svc_di
        MSR     CPSR_c, #SVC_MODE | FLAG_I
        MEND
        
        MACRO
        mode_irq_di
        MSR     CPSR_c, #IRQ_MODE | FLAG_I
        MEND

; ********************************************************
; *                  Global Symbols
; ********************************************************
        
        EXPORT LowlevelInit
        EXPORT SVC_Stack_Base
        EXPORT SVC_Stack_Limit
        EXPORT IRQ_Stack_Base
        EXPORT IRQ_Stack_Limit
        
; ********************************************************
; *                  External Symbols
; ********************************************************

; Information about image layout:

        IMPORT |Image$$RO$$Limit|
        IMPORT |Image$$RW$$Base|
        IMPORT |Image$$ZI$$Base|
        IMPORT |Image$$ZI$$Limit|
        

; ********************************************************
; *       Define Stacks here
; ********************************************************

        AREA SVCSTACK, NOINIT
SVC_Stack_Base
        % SVC_STACK_SIZE
SVC_Stack_Limit

        AREA IRQSTACK, NOINIT
IRQ_Stack_Base
        % IRQ_STACK_SIZE
IRQ_Stack_Limit


; ********************************************************
; *         init C memory
; ********************************************************

        AREA |C$$code|, CODE, READONLY
        ENTRY
LowlevelInit
        LDR R0, =|Image$$RO$$Limit|
        LDR R1, =|Image$$RW$$Base|
        LDR R3, =|Image$$ZI$$Base|
        CMP R0, R1
        BEQ nodata
init_data
        CMP R1,R3
        LDRCC R2, [R0], #4
        STRCC R2, [R1], #4
        BCC init_data
nodata
        LDR R1, =|Image$$ZI$$Limit|
        MOV R2, #0
clear_bss
        CMP R3, R1
        STRCC R2, [R3], #4
        BCC clear_bss
        
; ********************************************************
; *         init stack pointers
; ********************************************************

        mode_irq_di                    
        LDR R13, =IRQ_Stack_Limit      ; init SP_irq
        mode_svc_di
        LDR R13, =SVC_Stack_Limit      ; init SP_svc

; ********************************************************
; *         jump to C_Entry function (main).
; ******************************************************** 
; 
        IMPORT C_Entry
        BL C_Entry
Halt
        B Halt
        END

        
